// SPDX-FileCopyrightText: Copyright 2025 go-swagger maintainers
// SPDX-License-Identifier: Apache-2.0

// Package generator generates the testify public API.
//
// It adapts the internal testify API from github.com/go-openapi/testify/v2/internal/assertions
// and generates the content in packages "assert" and "require".
package generator
