// SPDX-FileCopyrightText: Copyright 2025 go-swagger maintainers
// SPDX-License-Identifier: Apache-2.0

// Package enable allows to register extra features.
//
// # Available extensions
//
//   - yaml: enable the YAML assertions
//   - colors: enable colorization of diff output
package enable
