// SPDX-FileCopyrightText: Copyright 2025 go-swagger maintainers
// SPDX-License-Identifier: Apache-2.0

// Package enable contains go modules with extra-dependencies.
//
// They allow extended capabilities to be added to testify by importing
// these packages specifically.
package enable
