// SPDX-FileCopyrightText: Copyright 2025 go-swagger maintainers
// SPDX-License-Identifier: Apache-2.0

// Package scanner exposes a golang source [Scanner].
//
// The [Scanner] scans the internal/assertions package.
//
// It extracts all testify helpers methods, types, and
// package-level variables and constants.
package scanner
