// SPDX-FileCopyrightText: Copyright 2025 go-swagger maintainers
// SPDX-License-Identifier: Apache-2.0

// Package colors exercises the colors feature (colorized output).
//
// It requires build tag "testcolorized" to actually run the test.
package colors
