// SPDX-FileCopyrightText: Copyright 2025 go-swagger maintainers
// SPDX-License-Identifier: Apache-2.0

// Package domains provides domain-based organization of assertions for documentation generation.
//
// This package handles the reorganization of assertion functions, types, variables, and constants
// from package-based organization (assert, require, internal/assertions) to domain-based organization
// (boolean, collection, error, etc.) for documentation purposes.
package domains
