// SPDX-FileCopyrightText: Copyright 2025 go-swagger maintainers
// SPDX-License-Identifier: Apache-2.0

// Package funcmaps provides template functions for code and documentation generation.
//
// This package contains all template helper functions used by both the code generator
// (for assert/require packages) and the documentation generator (for markdown docs).
//
// The main entry point is [FuncMap], which returns a complete [template.FuncMap]
// with all available functions registered.
package funcmaps
