// SPDX-FileCopyrightText: Copyright 2025 go-swagger maintainers
// SPDX-License-Identifier: Apache-2.0

// Package unsafetests exists only to isolate tests that reference the [unsafe] package.
//
// The tests in this package are totally safe.
package unsafetests
