// SPDX-FileCopyrightText: Copyright 2025 go-swagger maintainers
// SPDX-License-Identifier: Apache-2.0

// Code generated with github.com/go-openapi/testify/codegen/v2; DO NOT EDIT.

package require

import (
	"testing"
)

func TestCallerInfof(t *testing.T) {
	t.Skip() // this function doesn't have tests yet
}

func TestHTTPBodyf(t *testing.T) {
	t.Skip() // this function doesn't have tests yet
}

func TestObjectsAreEqualf(t *testing.T) {
	t.Skip() // this function doesn't have tests yet
}

func TestObjectsAreEqualValuesf(t *testing.T) {
	t.Skip() // this function doesn't have tests yet
}
