// SPDX-FileCopyrightText: Copyright 2025 go-swagger maintainers
// SPDX-License-Identifier: Apache-2.0

// Package yaml enables the [YAMLEq] capability in testify.
//
// # Usage
//
// To enable YAML assertion features, use a blank import like so:
//
//	import (
//			_ "github.com/go-openapi/testify/enable/yaml/v2"
//	)
//
// [YAMLEq]: https://pkg.go.dev/github.com/go-openapi/testify/v2/assert#YAMLEq
package yaml
